MfpPlan

Struct MfpPlan 

pub struct MfpPlan<E: OptimizableExpr = MirScalarExpr> {
    pub(crate) mfp: SafeMfpPlan<E>,
    pub(crate) lower_bounds: Vec<E>,
    pub(crate) upper_bounds: Vec<E>,
}
Predicates partitioned into temporal and non-temporal.

Temporal predicates require some recognition to determine their structure, and it is best to do that once and re-use the results.

There are restrictions on the temporal predicates we currently support. They must directly constrain MzNow from below or above, by expressions that do not themselves contain MzNow. Conjunctions of such constraints are also ok.

Fields

mfp: SafeMfpPlan<E>

Normal predicates to evaluate on &[Datum] and expect Ok(Datum::True).

lower_bounds: Vec<E>

Expressions that when evaluated lower-bound or equal (<=) MzNow.

upper_bounds: Vec<E>

Expressions that when evaluated strictly upper-bound MzNow.

Implementations

Source

impl<E: OptimizableExpr> MfpPlan<E>

pub fn from_parts( mfp: SafeMfpPlan<E>, lower_bounds: Vec<E>, upper_bounds: Vec<E>, ) -> Self

Construct an MfpPlan from its components.

pub fn into_parts(self) -> (SafeMfpPlan<E>, Vec<E>, Vec<E>)

Deconstruct into components.

pub fn safe_mfp(&self) -> &SafeMfpPlan<E>

Access the inner SafeMfpPlan.

pub fn as_parts(&self) -> (&SafeMfpPlan<E>, &[E], &[E])

Borrow all parts: (safe_mfp, lower_bounds, upper_bounds).

pub fn create_from(mfp: MapFilterProject<E>) -> Result<Self, String>

Partitions predicates into non-temporal, and lower and upper temporal bounds.

The first returned list is of predicates that do not contain mz_now. The second and third returned lists contain expressions that, once evaluated, lower and upper bound the validity interval of a record, respectively. These second two lists are populated only by binary expressions of the form

mz_now cmp_op expr

where cmp_op is a comparison operator and expr does not contain mz_now.

If any unsupported expression is found, for example one that uses mz_now in an unsupported position, an error is returned.

pub fn is_identity(&self) -> bool

Indicates if the planned MapFilterProject emits exactly its inputs as outputs.

pub fn has_temporal_bounds(&self) -> bool

Returns true if the plan contains temporal bounds (i.e., predicates involving mz_now()).

pub fn take(&mut self) -> Self

Returns self, and leaves behind an identity operator that acts on its output.

pub fn into_nontemporal(self) -> Result<SafeMfpPlan<E>, Self>

Attempt to convert self into a non-temporal MapFilterProject plan.

If that is not possible, the original instance is returned as an error.

pub fn iter_nontemporal_exprs(&mut self) -> impl Iterator<Item = &mut E>

Returns an iterator over mutable references to all non-temporal scalar expressions in the plan.

The order of iteration is unspecified.

pub fn ignores_input(&self) -> bool

Indicates that Self ignores its input to the extent that it can be evaluated on &[].

At the moment, this is only true if it projects away all columns and applies no filters, but it could be extended to plans that produce literals independent of the input.

Source

impl<E: OptimizableExpr + Eval> MfpPlan<E>

pub fn evaluate<'b, 'a: 'b, Err: From<EvalError>, V: Fn(&Timestamp) -> bool>( &'a self, datums: &'b mut Vec<Datum<'a>>, arena: &'a RowArena, time: Timestamp, diff: Diff, valid_time: V, row_builder: &mut Row, ) -> impl Iterator<Item = Result<(Row, Timestamp, Diff), (Err, Timestamp, Diff)>> + use<Err, V, E>

Evaluate the predicates, temporal and non-, and return times and differences for data.

If self contains only non-temporal predicates, the result will either be (time, diff), or an evaluation error. If self contains temporal predicates, the results can be times that are greater than the input time, and may contain negated diff` values.

The row_builder is not cleared first, but emptied if the function returns an iterator with any Ok(_) element.

pub fn could_error(&self) -> bool

Returns true if evaluation could introduce an error on non-error inputs.