SafeMfpPlan

Struct SafeMfpPlan 

pub struct SafeMfpPlan<E: OptimizableExpr = MirScalarExpr> {
    pub(crate) mfp: MapFilterProject<E>,
}
A wrapper type which indicates it is safe to simply evaluate all expressions.

Fields

mfp: MapFilterProject<E>

Implementations

Source

impl<E: OptimizableExpr> SafeMfpPlan<E>

pub fn from_mfp(mfp: MapFilterProject<E>) -> Self

Wrap a MapFilterProject in a SafeMfpPlan.

pub fn into_mfp(self) -> MapFilterProject<E>

Unwrap the inner MapFilterProject.

pub fn permute_fn<F>(&mut self, remap: F, new_arity: usize)
where F: Fn(usize) -> usize,

Remaps references to input columns according to remap.

Leaves other column references, e.g. to newly mapped columns, unchanged.

pub fn is_identity(&self) -> bool

Returns true when Self is the identity.

Source

impl<E: OptimizableExpr + Eval> SafeMfpPlan<E>

pub fn evaluate_into<'a, 'row>( &'a self, datums: &mut Vec<Datum<'a>>, arena: &'a RowArena, row_buf: &'row mut Row, ) -> Result<Option<&'row Row>, EvalError>

Evaluates the linear operator on a supplied list of datums.

The arguments are the initial datums associated with the row, and an appropriately lifetimed arena for temporary allocations needed by scalar evaluation.

An Ok result will either be None if any predicate did not evaluate to Datum::True, or the values of the columns listed by self.projection if all predicates passed. If an error occurs in the evaluation it is returned as an Err variant. As the evaluation exits early with failed predicates, it may miss some errors that would occur later in evaluation.

The row is not cleared first, but emptied if the function returns `Ok(Some(row)).

pub fn evaluate_iter<'b, 'a: 'b>( &'a self, datums: &'b mut Vec<Datum<'a>>, arena: &'a RowArena, ) -> Result<Option<impl Iterator<Item = Datum<'a>> + 'b>, EvalError>

A version of evaluate which produces an iterator over Datum as output.

This version can be useful when one wants to capture the resulting datums without packing and then unpacking a row.

pub fn evaluate_inner<'b, 'a: 'b>( &'a self, datums: &'b mut Vec<Datum<'a>>, arena: &'a RowArena, ) -> Result<bool, EvalError>

Populates datums with self.expressions and tests self.predicates.

This does not apply self.projection, which is up to the calling method.

pub fn could_error(&self) -> bool

Returns true if evaluation could introduce an error on non-error inputs.

Methods from Deref<Target = MapFilterProject<E>>

pub fn is_identity(&self) -> bool

True if the operator describes the identity transformation.

pub fn as_map_filter_project(&self) -> (Vec<E>, Vec<E>, Vec<usize>)

As the arguments to Map, Filter, and Project operators.

In principle, this operator can be implemented as a sequence of more elemental operators, likely less efficiently.

pub fn has_temporal_predicates(&self) -> bool

Returns true if any predicate in this MFP contains a temporal expression (mz_now()).

pub fn demand(&self) -> BTreeSet<usize>

Lists input columns whose values are used in outputs.

You can use BTreeSet::last() to extract the maximum demanded column from the set.

It is entirely appropriate to determine the demand of an instance and then both apply a projection to the subject of the instance and self.permute this instance.

pub fn size(&self) -> usize

Total expression sizes across all expressions.